@@ -1,16 +1,18 @@
 package targetconfigcontroller
 
 import (
+	"bytes"
 	"crypto/x509"
+	"encoding/json"
 	"fmt"
 	"reflect"
+	"strings"
 	"time"
 
-	"k8s.io/klog"
-
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
 	"k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -21,12 +23,11 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/util/cert"
 	"k8s.io/client-go/util/workqueue"
+	"k8s.io/klog"
 
 	operatorv1 "github.com/openshift/api/operator/v1"
-	operatorv1client "github.com/openshift/client-go/operator/clientset/versioned/typed/operator/v1"
-	operatorv1informers "github.com/openshift/client-go/operator/informers/externalversions/operator/v1"
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/operatorclient"
-	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/v311_00_assets"
+	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/operator/v411_00_assets"
 	"github.com/openshift/cluster-kube-controller-manager-operator/pkg/version"
 	"github.com/openshift/library-go/pkg/crypto"
 	"github.com/openshift/library-go/pkg/operator/events"
@@ -43,8 +44,7 @@ type TargetConfigController struct {
 	targetImagePullSpec   string
 	operatorImagePullSpec string
 
-	operatorConfigClient operatorv1client.KubeControllerManagersGetter
-	operatorClient       v1helpers.StaticPodOperatorClient
+	operatorClient v1helpers.StaticPodOperatorClient
 
 	kubeClient      kubernetes.Interface
 	configMapLister corev1listers.ConfigMapLister
@@ -58,9 +58,7 @@ type TargetConfigController struct {
 func NewTargetConfigController(
 	targetImagePullSpec, operatorImagePullSpec string,
 	kubeInformersForNamespaces v1helpers.KubeInformersForNamespaces,
-	operatorConfigInformer operatorv1informers.KubeControllerManagerInformer,
 	namespacedKubeInformers informers.SharedInformerFactory,
-	operatorConfigClient operatorv1client.KubeControllerManagersGetter,
 	operatorClient v1helpers.StaticPodOperatorClient,
 	kubeClient kubernetes.Interface,
 	eventRecorder events.Recorder,
@@ -69,17 +67,16 @@ func NewTargetConfigController(
 		targetImagePullSpec:   targetImagePullSpec,
 		operatorImagePullSpec: operatorImagePullSpec,
 
-		configMapLister:      kubeInformersForNamespaces.ConfigMapLister(),
-		secretLister:         kubeInformersForNamespaces.SecretLister(),
-		operatorConfigClient: operatorConfigClient,
-		operatorClient:       operatorClient,
-		kubeClient:           kubeClient,
-		eventRecorder:        eventRecorder.WithComponentSuffix("target-config-controller"),
+		configMapLister: kubeInformersForNamespaces.ConfigMapLister(),
+		secretLister:    kubeInformersForNamespaces.SecretLister(),
+		operatorClient:  operatorClient,
+		kubeClient:      kubeClient,
+		eventRecorder:   eventRecorder.WithComponentSuffix("target-config-controller"),
 
 		queue: workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "TargetConfigController"),
 	}
 
-	operatorConfigInformer.Informer().AddEventHandler(c.eventHandler())
+	operatorClient.Informer().AddEventHandler(c.eventHandler())
 	namespacedKubeInformers.Rbac().V1().Roles().Informer().AddEventHandler(c.eventHandler())
 	namespacedKubeInformers.Rbac().V1().RoleBindings().Informer().AddEventHandler(c.eventHandler())
 	namespacedKubeInformers.Core().V1().ConfigMaps().Informer().AddEventHandler(c.eventHandler())
@@ -98,12 +95,12 @@ func NewTargetConfigController(
 }
 
 func (c TargetConfigController) sync() error {
-	operatorConfig, err := c.operatorConfigClient.KubeControllerManagers().Get("cluster", metav1.GetOptions{})
+	operatorSpec, _, _, err := c.operatorClient.GetStaticPodOperatorStateWithQuorum()
 	if err != nil {
 		return err
 	}
 
-	switch operatorConfig.Spec.ManagementState {
+	switch operatorSpec.ManagementState {
 	case operatorv1.Managed:
 	case operatorv1.Unmanaged:
 		return nil
@@ -111,11 +108,17 @@ func (c TargetConfigController) sync() error {
 		// TODO probably just fail
 		return nil
 	default:
-		c.eventRecorder.Warningf("ManagementStateUnknown", "Unrecognized operator management state %q", operatorConfig.Spec.ManagementState)
+		c.eventRecorder.Warningf("ManagementStateUnknown", "Unrecognized operator management state %q", operatorSpec.ManagementState)
 		return nil
 	}
 
-	requeue, err := createTargetConfigController(c, c.eventRecorder, operatorConfig)
+	// block until config is observed and specific paths are present
+	if err := isRequiredConfigPresent(operatorSpec.ObservedConfig.Raw); err != nil {
+		c.eventRecorder.Warning("ConfigMissing", err.Error())
+		return err
+	}
+
+	requeue, err := createTargetConfigController(c, c.eventRecorder, operatorSpec)
 	if err != nil {
 		return err
 	}
@@ -126,19 +129,51 @@ func (c TargetConfigController) sync() error {
 	return nil
 }
 
+func isRequiredConfigPresent(config []byte) error {
+	if len(config) == 0 {
+		return fmt.Errorf("no observedConfig")
+	}
+
+	existingConfig := map[string]interface{}{}
+	if err := json.NewDecoder(bytes.NewBuffer(config)).Decode(&existingConfig); err != nil {
+		return fmt.Errorf("error parsing config, %v", err)
+	}
+
+	requiredPaths := [][]string{
+		{"extendedArguments", "cluster-name"},
+	}
+	for _, requiredPath := range requiredPaths {
+		configVal, found, err := unstructured.NestedFieldNoCopy(existingConfig, requiredPath...)
+		if err != nil {
+			return fmt.Errorf("error reading %v from config, %v", strings.Join(requiredPath, "."), err)
+		}
+		if !found {
+			return fmt.Errorf("%v missing from config", strings.Join(requiredPath, "."))
+		}
+		if configVal == nil {
+			return fmt.Errorf("%v null in config", strings.Join(requiredPath, "."))
+		}
+		if configValSlice, ok := configVal.([]interface{}); ok && len(configValSlice) == 0 {
+			return fmt.Errorf("%v empty in config", strings.Join(requiredPath, "."))
+		}
+		if configValString, ok := configVal.(string); ok && len(configValString) == 0 {
+			return fmt.Errorf("%v empty in config", strings.Join(requiredPath, "."))
+		}
+	}
+	return nil
+}
+
 // createTargetConfigController takes care of synchronizing (not upgrading) the thing we're managing.
-func createTargetConfigController(c TargetConfigController, recorder events.Recorder, operatorConfig *operatorv1.KubeControllerManager) (bool, error) {
+func createTargetConfigController(c TargetConfigController, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (bool, error) {
 	errors := []error{}
 
-	directResourceResults := resourceapply.ApplyDirectly(c.kubeClient, c.eventRecorder, v311_00_assets.Asset,
-		"v3.11.0/kube-controller-manager/ns.yaml",
-		"v3.11.0/kube-controller-manager/kubeconfig-cert-syncer.yaml",
-		"v3.11.0/kube-controller-manager/kubeconfig-cm.yaml",
-		"v3.11.0/kube-controller-manager/leader-election-rolebinding.yaml",
-		"v3.11.0/kube-controller-manager/leader-election-kube-controller-manager-role-kube-system.yaml",
-		"v3.11.0/kube-controller-manager/leader-election-kube-controller-manager-rolebinding-kube-system.yaml",
-		"v3.11.0/kube-controller-manager/svc.yaml",
-		"v3.11.0/kube-controller-manager/sa.yaml",
+	directResourceResults := resourceapply.ApplyDirectly(c.kubeClient, c.eventRecorder, v411_00_assets.Asset,
+		"v4.1.0/kube-controller-manager/ns.yaml",
+		"v4.1.0/kube-controller-manager/kubeconfig-cert-syncer.yaml",
+		"v4.1.0/kube-controller-manager/kubeconfig-cm.yaml",
+		"v4.1.0/kube-controller-manager/leader-election-rolebinding.yaml",
+		"v4.1.0/kube-controller-manager/svc.yaml",
+		"v4.1.0/kube-controller-manager/sa.yaml",
 	)
 	for _, currResult := range directResourceResults {
 		if currResult.Error != nil {
@@ -146,7 +181,7 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 		}
 	}
 
-	_, _, err := manageKubeControllerManagerConfig(c.kubeClient.CoreV1(), recorder, operatorConfig)
+	_, _, err := manageKubeControllerManagerConfig(c.kubeClient.CoreV1(), recorder, operatorSpec)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap", err))
 	}
@@ -158,15 +193,18 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/csr-controller-ca", err))
 	}
-	_, _, err = manageCSRSigner(c.secretLister, c.kubeClient.CoreV1(), recorder)
+	_, requeueDelay, _, err := manageCSRSigner(c.secretLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "secrets/csr-signer", err))
 	}
+	if requeueDelay > 0 {
+		c.queue.AddAfter(workQueueKey, requeueDelay)
+	}
 	_, _, err = manageServiceAccountCABundle(c.configMapLister, c.kubeClient.CoreV1(), recorder)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/serviceaccount-ca", err))
 	}
-	_, _, err = managePod(c.kubeClient.CoreV1(), c.kubeClient.CoreV1(), recorder, operatorConfig, c.targetImagePullSpec, c.operatorImagePullSpec)
+	_, _, err = managePod(c.kubeClient.CoreV1(), c.kubeClient.CoreV1(), recorder, operatorSpec, c.targetImagePullSpec, c.operatorImagePullSpec)
 	if err != nil {
 		errors = append(errors, fmt.Errorf("%q: %v", "configmap/kube-controller-manager-pod", err))
 	}
@@ -195,18 +233,18 @@ func createTargetConfigController(c TargetConfigController, recorder events.Reco
 	return false, nil
 }
 
-func manageKubeControllerManagerConfig(client corev1client.ConfigMapsGetter, recorder events.Recorder, operatorConfig *operatorv1.KubeControllerManager) (*corev1.ConfigMap, bool, error) {
-	configMap := resourceread.ReadConfigMapV1OrDie(v311_00_assets.MustAsset("v3.11.0/kube-controller-manager/cm.yaml"))
-	defaultConfig := v311_00_assets.MustAsset("v3.11.0/kube-controller-manager/defaultconfig.yaml")
-	requiredConfigMap, _, err := resourcemerge.MergeConfigMap(configMap, "config.yaml", nil, defaultConfig, operatorConfig.Spec.ObservedConfig.Raw, operatorConfig.Spec.UnsupportedConfigOverrides.Raw)
+func manageKubeControllerManagerConfig(client corev1client.ConfigMapsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec) (*corev1.ConfigMap, bool, error) {
+	configMap := resourceread.ReadConfigMapV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/cm.yaml"))
+	defaultConfig := v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/defaultconfig.yaml")
+	requiredConfigMap, _, err := resourcemerge.MergeConfigMap(configMap, "config.yaml", nil, defaultConfig, operatorSpec.ObservedConfig.Raw, operatorSpec.UnsupportedConfigOverrides.Raw)
 	if err != nil {
 		return nil, false, err
 	}
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
-func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter corev1client.SecretsGetter, recorder events.Recorder, operatorConfig *operatorv1.KubeControllerManager, imagePullSpec, operatorImagePullSpec string) (*corev1.ConfigMap, bool, error) {
-	required := resourceread.ReadPodV1OrDie(v311_00_assets.MustAsset("v3.11.0/kube-controller-manager/pod.yaml"))
+func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter corev1client.SecretsGetter, recorder events.Recorder, operatorSpec *operatorv1.StaticPodOperatorSpec, imagePullSpec, operatorImagePullSpec string) (*corev1.ConfigMap, bool, error) {
+	required := resourceread.ReadPodV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/pod.yaml"))
 	// TODO: If the image pull spec is not specified, the "${IMAGE}" will be used as value and the pod will fail to start.
 	images := map[string]string{
 		"${IMAGE}":          imagePullSpec,
@@ -232,7 +270,7 @@ func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter cor
 	}
 
 	var v int
-	switch operatorConfig.Spec.LogLevel {
+	switch operatorSpec.LogLevel {
 	case operatorv1.Normal:
 		v = 2
 	case operatorv1.Debug:
@@ -253,9 +291,9 @@ func managePod(configMapsGetter corev1client.ConfigMapsGetter, secretsGetter cor
 		required.Spec.Containers[0].Args = append(required.Spec.Containers[0].Args, "--tls-private-key-file=/etc/kubernetes/static-pod-resources/secrets/serving-cert/tls.key")
 	}
 
-	configMap := resourceread.ReadConfigMapV1OrDie(v311_00_assets.MustAsset("v3.11.0/kube-controller-manager/pod-cm.yaml"))
+	configMap := resourceread.ReadConfigMapV1OrDie(v411_00_assets.MustAsset("v4.1.0/kube-controller-manager/pod-cm.yaml"))
 	configMap.Data["pod.yaml"] = resourceread.WritePodV1OrDie(required)
-	configMap.Data["forceRedeploymentReason"] = operatorConfig.Spec.ForceRedeploymentReason
+	configMap.Data["forceRedeploymentReason"] = operatorSpec.ForceRedeploymentReason
 	configMap.Data["version"] = version.Get().String()
 	return resourceapply.ApplyConfigMap(configMapsGetter, recorder, configMap)
 }
@@ -291,32 +329,42 @@ func manageCSRCABundle(lister corev1listers.ConfigMapLister, client corev1client
 	return resourceapply.ApplyConfigMap(client, recorder, requiredConfigMap)
 }
 
-func manageCSRSigner(lister corev1listers.SecretLister, client corev1client.SecretsGetter, recorder events.Recorder) (*corev1.Secret, bool, error) {
+func manageCSRSigner(lister corev1listers.SecretLister, client corev1client.SecretsGetter, recorder events.Recorder) (*corev1.Secret, time.Duration, bool, error) {
 	// get the certkey pair we will sign with. We're going to add the cert to a ca bundle so we can recognize the chain it signs back to the signer
 	csrSigner, err := lister.Secrets(operatorclient.OperatorNamespace).Get("csr-signer")
 	if apierrors.IsNotFound(err) {
-		return nil, false, nil
+		return nil, 0, false, nil
 	}
 	if err != nil {
-		return nil, false, err
+		return nil, 0, false, err
 	}
 
 	// the CSR signing controller only accepts a single cert.  make sure we only ever have one (not multiple to construct a larger chain)
 	signingCert := csrSigner.Data["tls.crt"]
 	if len(signingCert) == 0 {
-		return nil, false, nil
+		return nil, 0, false, nil
 	}
 	signingKey := csrSigner.Data["tls.key"]
 	if len(signingCert) == 0 {
-		return nil, false, nil
+		return nil, 0, false, nil
 	}
 	signingCertKeyPair, err := crypto.GetCAFromBytes(signingCert, signingKey)
 	if err != nil {
-		return nil, false, err
+		return nil, 0, false, err
 	}
 	certBytes, err := crypto.EncodeCertificates(signingCertKeyPair.Config.Certs[0])
 	if err != nil {
-		return nil, false, err
+		return nil, 0, false, err
+	}
+
+	// make sure we wait five minutes to propagate the change to other components, like kas for trust
+	useAfter := signingCertKeyPair.Config.Certs[0].NotBefore.Add(5 * time.Minute)
+	now := time.Now()
+	if useAfter.Before(now) {
+		// if we have something and it's not expired (yeah that check is missing here), delay
+		if _, err := client.Secrets(operatorclient.TargetNamespace).Get("csr-signer", metav1.GetOptions{}); err == nil {
+			return nil, useAfter.Sub(now) + 10*time.Second, false, nil
+		}
 	}
 
 	csrSigner = &corev1.Secret{
@@ -326,7 +374,8 @@ func manageCSRSigner(lister corev1listers.SecretLister, client corev1client.Secr
 			"tls.key": []byte(signingKey),
 		},
 	}
-	return resourceapply.ApplySecret(client, recorder, csrSigner)
+	secret, modified, err := resourceapply.ApplySecret(client, recorder, csrSigner)
+	return secret, 0, modified, err
 }
 
 func manageCSRIntermediateCABundle(lister corev1listers.SecretLister, client corev1client.ConfigMapsGetter, recorder events.Recorder) (*corev1.ConfigMap, bool, error) {
